@@ -9,32 +9,18 @@ from django.test import TestCase
 from django.test.utils import override_settings
 
 from custard.conf import CUSTOM_TYPE_TEXT, CUSTOM_TYPE_INTEGER, settings
-from custard.models import custom
-from custard.forms import CustomFieldModelBaseForm
+from custard.builder import CustomFieldsBuilder
 from custard.utils import import_class
 
 from .models import (SimpleModelWithManager, SimpleModelWithoutManager,
-    CustomFieldsModel, CustomValuesModel)
+    CustomFieldsModel, CustomValuesModel, builder)
 
 
 #==============================================================================
-class SimpleModelWithManagerForm(CustomFieldModelBaseForm):
+class SimpleModelWithManagerForm(builder.create_modelform()):
     class Meta:
         model = SimpleModelWithManager
 
-    def get_fields_for_content_type(self, content_type):
-        return CustomFieldsModel.objects.filter(content_type=content_type)
-
-    def search_value_for_field(self, field, content_type, object_id):
-        return CustomValuesModel.objects.filter(custom_field=field,
-                                                content_type=content_type,
-                                                object_id=object_id)
-
-    def create_value_for_field(self, field, object_id, value):
-        return CustomValuesModel(custom_field=field,
-                                 object_id=object_id,
-                                 value=value)
-
 #class ExampleAdmin(admin.ModelAdmin):
 #    form = ExampleForm
 #    search_fields = ('name',)
@@ -74,19 +60,24 @@ class CustomModelsTestCase(TestCase):
         CustomFieldsModel.objects.all().delete()
 
     def test_import_class(self):
-        self.assertEqual(import_class('custard.forms.CustomFieldModelBaseForm'), CustomFieldModelBaseForm)
+        self.assertEqual(import_class('custard.builder.CustomFieldsBuilder'), CustomFieldsBuilder)
 
     @override_settings(CUSTOM_CONTENT_TYPES=['simplemodelwithmanager'])
     def test_field_creation(self):
-        class TestCustomFieldsModel(custom.create_fields()):
+        builder2 = CustomFieldsBuilder('tests.CustomFieldsModel',
+                                       'tests.CustomValuesModel',
+                                       settings.CUSTOM_CONTENT_TYPES)
+
+        class TestCustomFieldsModel(builder2.create_fields()):
             class Meta:
                 app_label = 'tests'
 
-        self.assertQuerysetEqual(ContentType.objects.filter(TestCustomFieldsModel.CONTENT_TYPES),
+        self.assertQuerysetEqual(ContentType.objects.filter(builder2.content_types_query),
                                  ContentType.objects.filter(Q(name__in=['simplemodelwithmanager'])))
 
     def test_mixin(self):
         self.assertIn(self.cf, self.obj.get_custom_fields())
+        self.assertIn(self.cf, SimpleModelWithManager.get_model_custom_fields())
         self.assertEqual(self.cf, self.obj.get_custom_field('text_field'))
 
         val = CustomValuesModel.objects.create(custom_field=self.cf, object_id=self.obj.pk)
@@ -175,12 +166,24 @@ class CustomModelsTestCase(TestCase):
 
     def test_get_formfield_for_field(self):
         with self.settings(CUSTOM_FIELD_TYPES={CUSTOM_TYPE_TEXT: 'django.forms.fields.EmailField'}):
-            form = SimpleModelWithManagerForm(data={}, instance=self.obj)
+            builder2 = CustomFieldsBuilder('tests.CustomFieldsModel', 'tests.CustomValuesModel')
+
+            class SimpleModelWithManagerForm2(builder2.create_modelform(field_types=settings.CUSTOM_FIELD_TYPES)):
+                class Meta:
+                    model = SimpleModelWithManager
+
+            form = SimpleModelWithManagerForm2(data={}, instance=self.obj)
             self.assertIsNotNone(form.get_formfield_for_field(self.cf))
             self.assertEqual(django.forms.fields.EmailField, form.get_formfield_for_field(self.cf).__class__)
 
     def test_get_widget_for_field(self):
         with self.settings(CUSTOM_WIDGET_TYPES={CUSTOM_TYPE_TEXT: 'django.forms.widgets.CheckboxInput'}):
-            form = SimpleModelWithManagerForm(data={}, instance=self.obj)
+            builder2 = CustomFieldsBuilder('tests.CustomFieldsModel', 'tests.CustomValuesModel')
+
+            class SimpleModelWithManagerForm2(builder2.create_modelform(widget_types=settings.CUSTOM_WIDGET_TYPES)):
+                class Meta:
+                    model = SimpleModelWithManager
+
+            form = SimpleModelWithManagerForm2(data={}, instance=self.obj)
             self.assertIsNotNone(form.get_widget_for_field(self.cf))
             self.assertEqual(django.forms.widgets.CheckboxInput, form.get_widget_for_field(self.cf).__class__)
